@@ -1,40 +1,41 @@
-use crate::session::*;
+use super::errors::MyError;
 use super::functions::*;
 use super::help::HELP;
 use super::nums::*;
 use super::state::*;
+use crate::session::*;
 use std::ffi::OsStr;
 // use clipboard::{ClipboardContext, ClipboardProvider};
 use log::debug;
 use log::error;
 use std::io::{stdin, stdout, Write};
 use std::path::{Path, PathBuf};
+use std::time::Instant;
 use termion::cursor::DetectCursorPos;
 use termion::event::Key;
 use termion::input::TermRead;
 use termion::raw::IntoRawMode;
 use termion::{clear, cursor, screen};
 
-pub fn run(arg: PathBuf) {
+pub fn run(arg: PathBuf) -> Result<(), MyError> {
     env_logger::init();
     debug!("starts initial setup.");
 
-    let mut config_dir = dirs::config_dir().unwrap_or_else(|| panic!("cannot read config dir."));
+    let mut config_dir = dirs::config_dir().unwrap_or_else(|| panic!("Cannot read config dir."));
     config_dir.push(FX_CONFIG_DIR);
     let config_file = config_dir.join(PathBuf::from(CONFIG_FILE));
     let trash_dir = config_dir.join(PathBuf::from(TRASH));
     make_config(&config_file, &trash_dir)
-        .unwrap_or_else(|_| panic!("cannot make config file or trash dir."));
+        .unwrap_or_else(|_| panic!("Cannot make config file or trash dir."));
     let session_file = config_dir.join(PathBuf::from(SESSION_FILE));
-    make_session(&session_file)
-        .unwrap_or_else(|_| panic!("cannot make session file."));
+    make_session(&session_file).unwrap_or_else(|_| panic!("Cannot make session file."));
 
     if !&arg.exists() {
         println!("Invalid path: {}", &arg.display());
-        return;
+        return Ok(());
     }
 
-    let (column, row) = termion::terminal_size().unwrap();
+    let (column, row) = termion::terminal_size()?;
     if column < 21 {
         error!("too small terminal size.");
         panic!("panic due to terminal size (less than 21 column).")
@@ -54,7 +55,7 @@ pub fn run(arg: PathBuf) {
         name_max_len: name_max,
         time_start_pos: time_start,
     };
-    state.current_dir = arg.canonicalize().unwrap();
+    state.current_dir = arg.canonicalize()?;
     state.update_list();
     state.trash_dir = trash_dir;
 
@@ -70,7 +71,7 @@ pub fn run(arg: PathBuf) {
     state.list_up(nums.skip);
 
     state.move_cursor(&nums, STARTING_POINT);
-    screen.flush().unwrap();
+    screen.flush()?;
 
     let mut p_memo_v: Vec<CursorMemo> = Vec::new();
     let mut c_memo_v: Vec<ChildMemo> = Vec::new();
@@ -78,7 +79,7 @@ pub fn run(arg: PathBuf) {
 
     'main: loop {
         let len = state.list.len();
-        let (_, y) = screen.cursor_pos().unwrap();
+        let (_, y) = screen.cursor_pos()?;
         let input = stdin.next();
 
         if let Some(Ok(key)) = input {
@@ -127,7 +128,7 @@ pub fn run(arg: PathBuf) {
                         print!("{}{}g", cursor::Goto(2, 2), clear::CurrentLine,);
                         print!("{}", cursor::Show);
 
-                        screen.flush().unwrap();
+                        screen.flush()?;
 
                         'top: loop {
                             let input = stdin.next();
@@ -179,8 +180,8 @@ pub fn run(arg: PathBuf) {
                         match item.file_type {
                             FileType::File | FileType::Symlink => {
                                 print!("{}", screen::ToAlternateScreen);
-                                if state.open_file(nums.index).is_err() {
-                                    print_warning("Cannot open file. Check your config!", y);
+                                if let Err(e) = state.open_file(nums.index) {
+                                    print_warning(e, y);
                                     continue;
                                 }
                                 print!("{}", screen::ToAlternateScreen);
@@ -340,12 +341,12 @@ pub fn run(arg: PathBuf) {
                     clear_and_show(&state.current_dir);
                     state.list_up(nums.skip);
                     state.move_cursor(&nums, y);
-                    screen.flush().unwrap();
+                    screen.flush()?;
 
                     let start_pos = nums.index;
 
                     loop {
-                        let (_, y) = screen.cursor_pos().unwrap();
+                        let (_, y) = screen.cursor_pos()?;
                         let input = stdin.next();
                         if let Some(Ok(key)) = input {
                             match key {
@@ -372,7 +373,7 @@ pub fn run(arg: PathBuf) {
                                         clear_and_show(&state.current_dir);
                                         state.list_up(nums.skip);
                                         state.move_cursor(&nums, y);
-                                        screen.flush().unwrap();
+                                        screen.flush()?;
                                     } else {
                                         nums.go_down();
 
@@ -388,7 +389,7 @@ pub fn run(arg: PathBuf) {
                                         clear_and_show(&state.current_dir);
                                         state.list_up(nums.skip);
                                         state.move_cursor(&nums, y + 1);
-                                        screen.flush().unwrap();
+                                        screen.flush()?;
                                     }
                                 }
 
@@ -411,7 +412,7 @@ pub fn run(arg: PathBuf) {
                                         clear_and_show(&state.current_dir);
                                         state.list_up(nums.skip);
                                         state.move_cursor(&nums, STARTING_POINT + 3);
-                                        screen.flush().unwrap();
+                                        screen.flush()?;
                                     } else {
                                         nums.go_up();
 
@@ -438,7 +439,7 @@ pub fn run(arg: PathBuf) {
                                         print!("{}{}g", cursor::Goto(2, 2), clear::CurrentLine,);
                                         print!("{}", cursor::Show);
 
-                                        screen.flush().unwrap();
+                                        screen.flush()?;
 
                                         'top_select: loop {
                                             let input = stdin.next();
@@ -496,38 +497,49 @@ pub fn run(arg: PathBuf) {
 
                                 Key::Char('d') => {
                                     print_info("Processing...", y);
-                                    screen.flush().unwrap();
+                                    let start = Instant::now();
+                                    screen.flush()?;
 
                                     state.registered.clear();
-                                    let iter = state.list.clone().into_iter();
-                                    let mut i = 0;
-                                    for item in iter {
-                                        if item.selected {
-                                            match item.file_type {
-                                                FileType::Directory => {
-                                                    if let Err(e) = state.remove_and_yank_dir(item)
-                                                    {
-                                                        print_warning(e, y);
-                                                        break;
-                                                    }
+                                    let mut count = 0;
+                                    let clone = state.list.clone();
+                                    let iter = clone.iter().filter(|item| item.selected);
+                                    let total_selected = iter.clone().count();
+                                    for (i, item) in iter.enumerate() {
+                                        print!(
+                                            " {}{}{}",
+                                            cursor::Goto(2, 2),
+                                            clear::CurrentLine,
+                                            display_count(i, total_selected)
+                                        );
+                                        match item.file_type {
+                                            FileType::Directory => {
+                                                if let Err(e) =
+                                                    state.remove_and_yank_dir(item.clone())
+                                                {
+                                                    print_warning(e, y);
+                                                    break;
                                                 }
-                                                FileType::File | FileType::Symlink => {
-                                                    if let Err(e) = state.remove_and_yank_file(item)
-                                                    {
-                                                        print_warning(e, y);
-                                                        break;
-                                                    }
+                                            }
+                                            FileType::File | FileType::Symlink => {
+                                                if let Err(e) =
+                                                    state.remove_and_yank_file(item.clone())
+                                                {
+                                                    print_warning(e, y);
+                                                    break;
                                                 }
                                             }
-                                            i += 1;
                                         }
+                                        count += 1;
                                     }
                                     clear_and_show(&state.current_dir);
                                     state.update_list();
                                     state.list_up(nums.skip);
 
-                                    let mut delete_message: String = i.to_string();
-                                    delete_message.push_str(" items deleted");
+                                    let duration = duration_to_string(start.elapsed());
+                                    let mut delete_message: String = count.to_string();
+                                    delete_message
+                                        .push_str(&format!(" items deleted [{}]", duration));
                                     print_info(delete_message, y);
                                     print!(" ");
 
@@ -573,7 +585,7 @@ pub fn run(arg: PathBuf) {
                                 }
                             }
                         }
-                        screen.flush().unwrap();
+                        screen.flush()?;
                     }
                 }
 
@@ -600,22 +612,28 @@ pub fn run(arg: PathBuf) {
                         print!("{}{}d", cursor::Goto(2, 2), clear::CurrentLine,);
                         print!("{}", cursor::Show);
 
-                        screen.flush().unwrap();
+                        screen.flush()?;
 
                         'delete: loop {
                             let input = stdin.next();
                             if let Some(Ok(key)) = input {
                                 match key {
                                     Key::Char('d') => {
+                                        print!("{}", cursor::Hide);
                                         print_info("Processing...", y);
-                                        screen.flush().unwrap();
+                                        let start = Instant::now();
+                                        screen.flush()?;
 
                                         state.registered.clear();
-                                        let item = state.get_item(nums.index).unwrap().clone();
+                                        let item = state.get_item(nums.index)?.clone();
                                         match item.file_type {
                                             FileType::Directory => {
+                                                print!(
+                                                    " {}{}1/1",
+                                                    cursor::Goto(2, 2),
+                                                    clear::CurrentLine
+                                                );
                                                 if let Err(e) = state.remove_and_yank_dir(item) {
-                                                    print!("{}", cursor::Hide);
                                                     print_warning(e, y);
                                                     state.move_cursor(&nums, y);
                                                     break 'delete;
@@ -624,7 +642,6 @@ pub fn run(arg: PathBuf) {
                                             FileType::File | FileType::Symlink => {
                                                 if let Err(e) = state.remove_and_yank_file(item) {
                                                     clear_and_show(&state.current_dir);
-                                                    print!("{}", cursor::Hide);
                                                     print_warning(e, y);
                                                     state.move_cursor(&nums, y);
                                                     break 'delete;
@@ -633,7 +650,6 @@ pub fn run(arg: PathBuf) {
                                         }
 
                                         clear_and_show(&state.current_dir);
-                                        print!("{}", cursor::Hide);
                                         state.update_list();
                                         state.list_up(nums.skip);
                                         let cursor_pos = if state.list.is_empty() {
@@ -644,7 +660,11 @@ pub fn run(arg: PathBuf) {
                                         } else {
                                             y
                                         };
-                                        print_info("1 item deleted", cursor_pos);
+                                        let duration = duration_to_string(start.elapsed());
+                                        print_info(
+                                            format!("1 item deleted [{}]", duration),
+                                            cursor_pos,
+                                        );
                                         state.move_cursor(&nums, cursor_pos);
                                         break 'delete;
                                     }
@@ -668,7 +688,7 @@ pub fn run(arg: PathBuf) {
                     print!("{}{}y", cursor::Goto(2, 2), clear::CurrentLine,);
                     print!("{}", cursor::Show);
 
-                    screen.flush().unwrap();
+                    screen.flush()?;
 
                     'yank: loop {
                         let input = stdin.next();
@@ -701,7 +721,8 @@ pub fn run(arg: PathBuf) {
                         continue;
                     }
                     print_info("Processing...", y);
-                    screen.flush().unwrap();
+                    let start = Instant::now();
+                    screen.flush()?;
 
                     if let Err(e) = state.put_items() {
                         print_warning(e, y);
@@ -712,8 +733,14 @@ pub fn run(arg: PathBuf) {
                     state.update_list();
                     state.list_up(nums.skip);
 
-                    let mut put_message: String = state.registered.len().to_string();
-                    put_message.push_str(" items inserted");
+                    let duration = duration_to_string(start.elapsed());
+                    let registered_len = state.registered.len();
+                    let mut put_message: String = registered_len.to_string();
+                    if registered_len == 1 {
+                        put_message.push_str(&format!(" item inserted [{}]", duration));
+                    } else {
+                        put_message.push_str(&format!(" items inserted [{}]", duration));
+                    }
                     print_info(put_message, y);
                     state.move_cursor(&nums, y);
                 }
@@ -733,11 +760,11 @@ pub fn run(arg: PathBuf) {
                         RIGHT_ARROW,
                         &rename.iter().collect::<String>(),
                     );
-                    screen.flush().unwrap();
+                    screen.flush()?;
 
                     loop {
                         let eow = rename.len() + 3;
-                        let (x, _) = screen.cursor_pos().unwrap();
+                        let (x, _) = screen.cursor_pos()?;
                         let input = stdin.next();
                         if let Some(Ok(key)) = input {
                             match key {
@@ -766,7 +793,6 @@ pub fn run(arg: PathBuf) {
                                 Key::Esc => {
                                     print!("{}", clear::CurrentLine);
                                     print!("{}{}", cursor::Goto(2, 2), DOWN_ARROW);
-                                    screen.flush().unwrap();
 
                                     print!("{}", cursor::Hide);
                                     state.move_cursor(&nums, y);
@@ -778,7 +804,6 @@ pub fn run(arg: PathBuf) {
                                         continue;
                                     };
                                     print!("{}", cursor::Left(1));
-                                    screen.flush().unwrap();
                                 }
 
                                 Key::Right => {
@@ -786,7 +811,6 @@ pub fn run(arg: PathBuf) {
                                         continue;
                                     };
                                     print!("{}", cursor::Right(1));
-                                    screen.flush().unwrap();
                                 }
 
                                 Key::Char(c) => {
@@ -801,8 +825,6 @@ pub fn run(arg: PathBuf) {
                                         &rename.iter().collect::<String>(),
                                         cursor::Goto(memo_x + 1, 2)
                                     );
-
-                                    screen.flush().unwrap();
                                 }
 
                                 Key::Backspace => {
@@ -820,12 +842,11 @@ pub fn run(arg: PathBuf) {
                                         &rename.iter().collect::<String>(),
                                         cursor::Goto(memo_x - 1, 2)
                                     );
-
-                                    screen.flush().unwrap();
                                 }
 
                                 _ => continue,
                             }
+                            screen.flush()?;
                         }
                     }
                 }
@@ -841,13 +862,13 @@ pub fn run(arg: PathBuf) {
                         RIGHT_ARROW
                     );
                     print!("{}", cursor::Show);
-                    screen.flush().unwrap();
+                    screen.flush()?;
 
                     let original_list = state.list.clone();
 
                     let mut keyword: Vec<char> = Vec::new();
                     loop {
-                        let (x, _) = screen.cursor_pos().unwrap();
+                        let (x, _) = screen.cursor_pos()?;
                         let keyword_len = keyword.len();
 
                         let input = stdin.next();
@@ -857,7 +878,7 @@ pub fn run(arg: PathBuf) {
                                     filtered = true;
                                     print!("{}", clear::CurrentLine);
                                     print!("{}{}", cursor::Goto(2, 2), DOWN_ARROW);
-                                    screen.flush().unwrap();
+                                    screen.flush()?;
 
                                     nums.reset();
                                     state.move_cursor(&nums, STARTING_POINT);
@@ -880,7 +901,6 @@ pub fn run(arg: PathBuf) {
                                         continue;
                                     }
                                     print!("{}", cursor::Left(1));
-                                    screen.flush().unwrap();
                                 }
 
                                 Key::Right => {
@@ -888,7 +908,6 @@ pub fn run(arg: PathBuf) {
                                         continue;
                                     }
                                     print!("{}", cursor::Right(1));
-                                    screen.flush().unwrap();
                                 }
 
                                 Key::Char(c) => {
@@ -915,8 +934,6 @@ pub fn run(arg: PathBuf) {
                                         &keyword.iter().collect::<String>(),
                                         cursor::Goto(memo_x + 1, 2)
                                     );
-
-                                    screen.flush().unwrap();
                                 }
 
                                 Key::Backspace => {
@@ -947,12 +964,11 @@ pub fn run(arg: PathBuf) {
                                         &keyword.iter().collect::<String>(),
                                         cursor::Goto(memo_x - 1, 2)
                                     );
-
-                                    screen.flush().unwrap();
                                 }
 
                                 _ => continue,
                             }
+                            screen.flush()?;
                         }
                     }
                     print!("{}", cursor::Hide);
@@ -963,11 +979,11 @@ pub fn run(arg: PathBuf) {
                     print!("{}", cursor::Show);
 
                     let mut command: Vec<char> = Vec::new();
-                    screen.flush().unwrap();
+                    screen.flush()?;
 
                     'command: loop {
                         let eow = command.len() + 2;
-                        let (x, _) = screen.cursor_pos().unwrap();
+                        let (x, _) = screen.cursor_pos()?;
                         let input = stdin.next();
                         if let Some(Ok(key)) = input {
                             match key {
@@ -1024,7 +1040,7 @@ pub fn run(arg: PathBuf) {
 
                                     if c == "empty" && args.is_empty() {
                                         print_warning(WHEN_EMPTY, y);
-                                        screen.flush().unwrap();
+                                        screen.flush()?;
 
                                         'empty: loop {
                                             let input = stdin.next();
@@ -1032,7 +1048,7 @@ pub fn run(arg: PathBuf) {
                                                 match key {
                                                     Key::Char('y') | Key::Char('Y') => {
                                                         print_info("Processing...", y);
-                                                        screen.flush().unwrap();
+                                                        screen.flush()?;
 
                                                         if let Err(e) = std::fs::remove_dir_all(
                                                             &state.trash_dir,
@@ -1077,7 +1093,7 @@ pub fn run(arg: PathBuf) {
                                     print!("{}", screen::ToAlternateScreen);
                                     if std::env::set_current_dir(&state.current_dir).is_err() {
                                         print!("{}", cursor::Hide,);
-                                        print_warning("cannot execute command", y);
+                                        print_warning("Cannot execute command", y);
                                         break 'command;
                                     }
                                     if std::process::Command::new(c).args(args).status().is_err() {
@@ -1088,7 +1104,7 @@ pub fn run(arg: PathBuf) {
                                         state.list_up(nums.skip);
 
                                         print!("{}", cursor::Hide,);
-                                        print_warning("cannot execute command", y);
+                                        print_warning("Cannot execute command", y);
                                         break 'command;
                                     }
                                     print!("{}", screen::ToAlternateScreen);
@@ -1115,7 +1131,6 @@ pub fn run(arg: PathBuf) {
                                         continue;
                                     };
                                     print!("{}", cursor::Left(1));
-                                    screen.flush().unwrap();
                                 }
 
                                 Key::Right => {
@@ -1123,7 +1138,6 @@ pub fn run(arg: PathBuf) {
                                         continue;
                                     };
                                     print!("{}", cursor::Right(1));
-                                    screen.flush().unwrap();
                                 }
 
                                 Key::Char(c) => {
@@ -1136,8 +1150,6 @@ pub fn run(arg: PathBuf) {
                                         &command.iter().collect::<String>(),
                                         cursor::Goto(x + 1, 2)
                                     );
-
-                                    screen.flush().unwrap();
                                 }
 
                                 Key::Backspace => {
@@ -1153,12 +1165,11 @@ pub fn run(arg: PathBuf) {
                                         &command.iter().collect::<String>(),
                                         cursor::Goto(x - 1, 2)
                                     );
-
-                                    screen.flush().unwrap();
                                 }
 
                                 _ => continue,
                             }
+                            screen.flush()?;
                         }
                     }
                 }
@@ -1168,7 +1179,7 @@ pub fn run(arg: PathBuf) {
                     print!("{}", cursor::Show);
 
                     let mut command: Vec<char> = vec!['Z'];
-                    screen.flush().unwrap();
+                    screen.flush()?;
 
                     'quit: loop {
                         let input = stdin.next();
@@ -1215,9 +1226,10 @@ pub fn run(arg: PathBuf) {
                 }
             }
         }
-        screen.flush().unwrap();
+        screen.flush()?;
     }
-    //When finishes, restore the cursor
+    //When exits, restore the cursor
     print!("{}", cursor::Restore);
-    state.write_session(session_file);
+    state.write_session(session_file)?;
+    Ok(())
 }
