@@ -1,14 +1,15 @@
-use super::session::*;
 use super::config::*;
+use super::errors::MyError;
 use super::functions::*;
 use super::nums::*;
+use super::session::*;
 use chrono::prelude::*;
 use std::collections::HashMap;
 use std::collections::HashSet;
 use std::ffi::OsString;
 use std::fs;
 use std::fs::DirEntry;
-use std::io::{Error, ErrorKind};
+use std::io::ErrorKind;
 use std::path::{Path, PathBuf};
 use std::process::{Command, ExitStatus};
 use termion::{clear, color, cursor, style};
@@ -126,13 +127,16 @@ impl State {
     pub fn new() -> Self {
         Default::default()
     }
-    pub fn get_item(&self, index: usize) -> Result<&ItemInfo, std::io::Error> {
-        self.list
-            .get(index)
-            .ok_or_else(|| Error::new(ErrorKind::NotFound, "cannot choose item."))
+    pub fn get_item(&self, index: usize) -> Result<&ItemInfo, MyError> {
+        self.list.get(index).ok_or_else(|| {
+            MyError::IoError(std::io::Error::new(
+                ErrorKind::NotFound,
+                "Cannot choose item.",
+            ))
+        })
     }
 
-    pub fn open_file(&self, index: usize) -> std::io::Result<ExitStatus> {
+    pub fn open_file(&self, index: usize) -> Result<ExitStatus, MyError> {
         let item = self.get_item(index)?;
         let path = &item.file_path;
         let map = &self.commands;
@@ -144,29 +148,31 @@ impl State {
                 match map.get(&ext) {
                     Some(command) => {
                         let mut ex = Command::new(command);
-                        ex.arg(path).status()
+                        ex.arg(path).status().map_err(MyError::IoError)
                     }
                     None => {
                         let mut ex = Command::new(&self.default);
-                        ex.arg(path).status()
+                        ex.arg(path).status().map_err(MyError::IoError)
                     }
                 }
             }
 
             None => {
                 let mut ex = Command::new(&self.default);
-                ex.arg(path).status()
+                ex.arg(path).status().map_err(MyError::IoError)
             }
         }
     }
 
-    pub fn remove_and_yank_file(&mut self, item: ItemInfo) -> std::io::Result<()> {
+    pub fn remove_and_yank_file(&mut self, item: ItemInfo) -> Result<(), MyError> {
         //prepare from and to for copy
         let from = &item.file_path;
 
         if item.file_type == FileType::Symlink && !from.exists() {
-            let _ = Command::new("rm").arg(from).status();
-            Ok(())
+            match Command::new("rm").arg(from).status() {
+                Ok(_) => Ok(()),
+                Err(e) => Err(MyError::IoError(e)),
+            }
         } else {
             let name = &item.file_name;
             let mut rename = Local::now().timestamp().to_string();
@@ -176,60 +182,96 @@ impl State {
             let to = self.trash_dir.join(&rename);
 
             //copy
-            std::fs::copy(from, &to)?;
+            if std::fs::copy(from, &to).is_err() {
+                return Err(MyError::FileCopyError {
+                    msg: format!("Cannot copy item: {:?}", from),
+                });
+            }
 
             self.to_registered_mut(&item, to, rename);
 
             //remove original
-            std::fs::remove_file(from)?;
+            if std::fs::remove_file(from).is_err() {
+                return Err(MyError::FileRemoveError {
+                    msg: format!("Cannot Remove item: {:?}", from),
+                });
+            }
 
             Ok(())
         }
     }
 
-    pub fn remove_and_yank_dir(&mut self, item: ItemInfo) -> std::io::Result<()> {
+    pub fn remove_and_yank_dir(&mut self, item: ItemInfo) -> Result<(), MyError> {
         let mut trash_name = String::new();
         let mut base: usize = 0;
         let mut trash_path: std::path::PathBuf = PathBuf::new();
         let mut target: PathBuf;
 
-        let mut i = 0;
-        for entry in walkdir::WalkDir::new(&item.file_path).sort_by_key(|x| x.path().to_path_buf())
+        let len = walkdir::WalkDir::new(&item.file_path).into_iter().count();
+        let unit = len / 5;
+        for (i, entry) in walkdir::WalkDir::new(&item.file_path)
+            .into_iter()
+            .enumerate()
         {
+            if i > unit * 4 {
+                print_process("[»»»»-]");
+            } else if i > unit * 3 {
+                print_process("[»»»--]");
+            } else if i > unit * 2 {
+                print_process("[»»---]");
+            } else if i > unit {
+                print_process("[»----]");
+            } else if i == 0 {
+                print_process(" [-----]");
+            }
             let entry = entry?;
+            let entry_path = entry.path();
             if i == 0 {
-                base = entry.path().iter().count();
+                base = entry_path.iter().count();
 
                 trash_name = chrono::Local::now().timestamp().to_string();
                 trash_name.push('_');
-                trash_name.push_str(entry.file_name().to_str().unwrap());
+                let file_name = entry.file_name().to_str();
+                if file_name == None {
+                    return Err(MyError::UTF8Error {
+                        msg: "Cannot convert filename to UTF8.".to_string(),
+                    });
+                }
+                trash_name.push_str(file_name.unwrap());
                 trash_path = self.trash_dir.join(&trash_name);
                 std::fs::create_dir(&self.trash_dir.join(&trash_path))?;
 
-                i += 1;
                 continue;
             } else {
-                target = entry.path().iter().skip(base).collect();
+                target = entry_path.iter().skip(base).collect();
                 target = trash_path.join(target);
                 if entry.file_type().is_dir() {
-                    std::fs::create_dir(&target)?;
+                    std::fs::create_dir_all(&target)?;
                     continue;
                 }
 
-                if let Some(parent) = entry.path().parent() {
+                if let Some(parent) = entry_path.parent() {
                     if !parent.exists() {
                         std::fs::create_dir(parent)?;
                     }
                 }
 
-                std::fs::copy(entry.path(), &target)?;
+                if std::fs::copy(entry_path, &target).is_err() {
+                    return Err(MyError::FileCopyError {
+                        msg: format!("Cannot copy item: {:?}", entry_path),
+                    });
+                }
             }
         }
 
         self.to_registered_mut(&item, trash_path, trash_name);
 
         //remove original
-        std::fs::remove_dir_all(&item.file_path)?;
+        if std::fs::remove_dir_all(&item.file_path).is_err() {
+            return Err(MyError::FileRemoveError {
+                msg: format!("Cannot Remove directory: {:?}", item.file_name),
+            });
+        }
 
         Ok(())
     }
@@ -254,14 +296,21 @@ impl State {
         }
     }
 
-    pub fn put_items(&mut self) -> std::io::Result<()> {
+    pub fn put_items(&mut self) -> Result<(), MyError> {
         //make HashSet<String> of file_name
         let mut name_set = HashSet::new();
         for item in self.list.iter() {
             name_set.insert(item.file_name.clone());
         }
 
-        for item in self.registered.clone().into_iter() {
+        let total_selected = self.registered.len();
+        for (i, item) in self.registered.clone().into_iter().enumerate() {
+            print!(
+                " {}{}{}",
+                cursor::Goto(2, 2),
+                clear::CurrentLine,
+                display_count(i, total_selected)
+            );
             match item.file_type {
                 FileType::Directory => {
                     self.put_dir(&item, &mut name_set)?;
@@ -274,32 +323,56 @@ impl State {
         Ok(())
     }
 
-    fn put_file(&mut self, item: &ItemInfo, name_set: &mut HashSet<String>) -> std::io::Result<()> {
+    fn put_file(&mut self, item: &ItemInfo, name_set: &mut HashSet<String>) -> Result<(), MyError> {
         if item.file_path.parent() == Some(&self.trash_dir) {
             let mut item = item.clone();
             let rename = item.file_name.chars().skip(11).collect();
             item.file_name = rename;
             let rename = rename_file(&item, name_set);
-            std::fs::copy(&item.file_path, &self.current_dir.join(&rename))?;
+            if std::fs::copy(&item.file_path, &self.current_dir.join(&rename)).is_err() {
+                return Err(MyError::FileCopyError {
+                    msg: format!("Cannot copy item: {:?}", &item.file_path),
+                });
+            }
             name_set.insert(rename);
         } else {
             let rename = rename_file(item, name_set);
-            std::fs::copy(&item.file_path, &self.current_dir.join(&rename))?;
+            if std::fs::copy(&item.file_path, &self.current_dir.join(&rename)).is_err() {
+                return Err(MyError::FileCopyError {
+                    msg: format!("Cannot copy item: {:?}", &item.file_path),
+                });
+            }
             name_set.insert(rename);
         }
         Ok(())
     }
 
-    fn put_dir(&mut self, buf: &ItemInfo, name_set: &mut HashSet<String>) -> std::io::Result<()> {
+    fn put_dir(&mut self, buf: &ItemInfo, name_set: &mut HashSet<String>) -> Result<(), MyError> {
         let mut base: usize = 0;
         let mut target: PathBuf = PathBuf::new();
         let original_path = &(buf).file_path;
 
-        let mut i = 0;
-        for entry in walkdir::WalkDir::new(&original_path).sort_by_key(|x| x.path().to_path_buf()) {
+        let len = walkdir::WalkDir::new(&original_path).into_iter().count();
+        let unit = len / 5;
+        for (i, entry) in walkdir::WalkDir::new(&original_path)
+            .into_iter()
+            .enumerate()
+        {
+            if i > unit * 4 {
+                print_process("[»»»»-]");
+            } else if i > unit * 3 {
+                print_process("[»»»--]");
+            } else if i > unit * 2 {
+                print_process("[»»---]");
+            } else if i > unit {
+                print_process("[»----]");
+            } else if i == 0 {
+                print_process(" [»----]");
+            }
             let entry = entry?;
+            let entry_path = entry.path();
             if i == 0 {
-                base = entry.path().iter().count();
+                base = entry_path.iter().count();
 
                 let parent = &original_path.parent().unwrap();
                 if parent == &self.trash_dir {
@@ -316,22 +389,25 @@ impl State {
                     name_set.insert(rename);
                 }
                 std::fs::create_dir(&target)?;
-                i += 1;
                 continue;
             } else {
-                let child: PathBuf = entry.path().iter().skip(base).collect();
+                let child: PathBuf = entry_path.iter().skip(base).collect();
                 let child = target.join(child);
 
                 if entry.file_type().is_dir() {
-                    std::fs::create_dir(child)?;
+                    std::fs::create_dir_all(child)?;
                     continue;
-                } else if let Some(parent) = entry.path().parent() {
+                } else if let Some(parent) = entry_path.parent() {
                     if !parent.exists() {
                         std::fs::create_dir(parent)?;
                     }
                 }
 
-                std::fs::copy(entry.path(), &child)?;
+                if std::fs::copy(entry_path, &child).is_err() {
+                    return Err(MyError::FileCopyError {
+                        msg: format!("Cannot copy item: {:?}", entry_path),
+                    });
+                }
             }
         }
         Ok(())
@@ -559,17 +635,6 @@ impl State {
                 );
 
                 if row_count == row - STARTING_POINT {
-                    print!(
-                        "  {}{}{}lines {}-{}({}){}{}",
-                        cursor::Left(2),
-                        color::Bg(color::LightWhite),
-                        color::Fg(color::Black),
-                        skip_number + 1,
-                        row - STARTING_POINT + skip_number,
-                        len,
-                        color::Bg(color::Reset),
-                        color::Fg(color::Reset)
-                    );
                     break;
                 } else {
                     self.print(i);
@@ -643,19 +708,19 @@ impl State {
         print!("{}>{}", cursor::Goto(1, y), cursor::Left(1));
     }
 
-    pub fn write_session(&self, session_path: PathBuf) {
-        let session = Session{
+    pub fn write_session(&self, session_path: PathBuf) -> Result<(), MyError> {
+        let session = Session {
             sort_by: self.sort_by.clone(),
-            show_hidden: self.show_hidden
+            show_hidden: self.show_hidden,
         };
-        let serialized = toml::to_string(&session).unwrap();
-        fs::write(&session_path, serialized)
-            .unwrap_or_else(|_| panic!("cannot write new session file."));
+        let serialized = toml::to_string(&session)?;
+        fs::write(&session_path, serialized)?;
+        Ok(())
     }
 }
 
-fn make_item(dir: fs::DirEntry) -> ItemInfo {
-    let path = dir.path();
+fn make_item(entry: fs::DirEntry) -> ItemInfo {
+    let path = entry.path();
     let metadata = &fs::symlink_metadata(&path);
 
     let time = match metadata {
@@ -683,10 +748,10 @@ fn make_item(dir: fs::DirEntry) -> ItemInfo {
         Err(_) => FileType::File,
     };
 
-    let name = dir
+    let name = entry
         .file_name()
         .into_string()
-        .unwrap_or_else(|_| panic!("failed to get file name."));
+        .unwrap_or_else(|_| "Invalid unicode name".to_string());
 
     let size = match metadata {
         Ok(metadata) => metadata.len(),
@@ -714,7 +779,7 @@ fn is_not_hidden(entry: &DirEntry) -> bool {
         .unwrap_or(false)
 }
 
-pub fn push_items(p: &Path, key: &SortKey, show_hidden: bool) -> Result<Vec<ItemInfo>, Error> {
+pub fn push_items(p: &Path, key: &SortKey, show_hidden: bool) -> Result<Vec<ItemInfo>, MyError> {
     let mut result = Vec::new();
     let mut dir_v = Vec::new();
     let mut file_v = Vec::new();
